// $Id$ -*- C++ -*-
// A class for manipulating breakpoint distance graphs.
//
// Copyright (C) 1997 Humberto Ortiz Zuazaga
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// See the file licence.html for a copy of the GNU GPL

#ifndef BRKGRAPH_H
#define BRKGRAPH_H

#include "map.h"

// Each marker is a node in the graph.
class MNode {
  int index;			// Index of marker in datafile.
  int weight;			// Marker informativity.

public:
  MNode(void) {index = -1; weight = 0;};
  MNode (int i) { index = i; weight = 0;};
  MNode (int i, int w) { index = i; weight = w;};
  
  void setindex(int i) { index = i;};
  int getindex(void) { return index; };

  void setweight(int w) { weight = w;};
  int getweight(void) { return weight; };
};

class DEdge {
  int weight;			// The breakpoint distance along the edge.

public:
  DEdge(void) {weight = 0;};
  DEdge (int d) { weight = d; };

  void setweight(int w) { weight = w;};
  int getweight(void) { return weight; };
};

class BrkGraph {
  // Data
  MNode *nodes;
  char **edges;
  int n;			// number of nodes

public:
  // Functions
  BrkGraph(class map &mapdata, int minbrk);
  BrkGraph::~BrkGraph(void);  
  int rdfs(int *, int, int, int);
  int number_of_nodes(void) {return n;};
  FRS_MIS(int *order, int cend, int tend, int best);
  XRLF_MIS(int *order, int cend, int tend, int best,
	   int TrialNum, int SetLim, int Candnum);
};

#endif /* BRKGRAPH_H */
