// $Id$
// Recursive depth first search for best marker ordering.

#include <limits.h>
#include <fstream.h>

#include <LEDA/basic.h>
#include <LEDA/array.h>

#include "map.h"

int rdfs(map *M, int *order, int depth, int len, int best);

int main (int argc, char *argv[]) {
  map mapdata;

  if (argc < 3) {
    cerr << "Usage: sa-local {mapdata} {sequence}" << endl;
    exit(1);
  }

  ifstream in(argv[1]);
  if (!in) { 
    perror ("Error opening map file"); 
    exit (1);
  }
  mapdata.read(in, KNOWNS);
  in.close();

  // Read in the sequence file, this should be in a function or class!
  ifstream seqin(argv[2]);
  array<int> sequence(mapdata.cols());
  if (!seqin) {
    perror ("Error opening sequence file"); 
    exit (1);
  }
  sequence.read(seqin);
  seqin.close();

  int length;
  for (length = 0; length <= sequence.high(); length++) {
    if (sequence[length] == 0) {
      break;
    }
  }

  int *seq = new int[length];
  for (int i = 0; i < length; i++) {
    seq[i] = sequence[i] - 1;
  }

  int best = INT_MAX;
  int *order = new int[length];

  if (4 == argc) {
    best = atoi(argv[3]);
  }

  for (int i = 0; i < length; i++) {
    order[i] = seq[i];
  }

  cout << "Trying to find a map with " << best << " or fewer breaks." << endl;
  cout << "Estimate is " << mapdata.bound(order, 0, length) << endl;
  rdfs(&mapdata, order, 0, length, best);
}

// M = marker and chromosome info, read-only.
// order = array holding marker indicies for tour and test markers.
// depth = size of tour
// len = size of order array
// best = the best complete tour so far.
// order[0..depth-1] holds the indicies of the tour markers.
// order[depth..len-1] holds the indicies of the test markers.

int rdfs(map *M, int *order, int depth, int len, int best) {
  int *neworder;
  int myscore;

  if (depth > len) {
    return best;
  }
   
  myscore = M->bound(order, depth, len);

  if (best < myscore) {
    return best;
  }

  // copy the partial tour.
  neworder = new int[len];
  for (int i = 0; i < depth; i ++) {
    neworder[i] = order[i];
  }

  int mark;

  for (int i = depth; i < len; i++) {
    neworder[depth] = order[i];
    mark = depth + 1;
    for (int j = depth; j < len; j++) {
      if (i != j) {
	neworder[mark++] = order[j];
      }
    }

    if (best >= myscore) {
      best = rdfs(M, neworder, depth+1, len, best);
    } else {
      break;
    }
  }
 
  delete neworder;

  if ((depth == len) && (myscore <= best)) {
    best = myscore;
    cout << "Score = " << myscore << endl;
    for (int i = 0; i < len; i++) {
      // cout << M->markers[order[i]].getname() << " ";
      cout << order[i] + 1 << " ";
    }
    cout << endl;
  }

  return best;
}
