// $Id: map.h 1.3 Tue, 25 Feb 1997 12:34:46 -0400 humberto $ -*- C++ -*-
// Class to hold a map data set.
//
// Copyright (C) 1995, 1996 Humberto Ortiz Zuazaga and Rosemarie Plaetke.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// See the file licence.html for a copy of the GNU GPL

#ifndef MAP_H
#define MAP_H

#include <iostream.h>

#include <LEDA/basic.h>

#include "marker.h"
#include "chromosome.h"

enum {KNOWNS, RETAINED};

class map {
private:
  // Data
  marker *markers;		// Array of markers
  chromosome *chromosomes;	// Array of clones
  string *PMU;			// Array of typing results
  int **Pair;			// Pairwise marker distances (breaks)
  int m;			// number of markers (columns)
  int n;			// number of chromosomes (rows)

  // Functions
  int pairscore (int i, int j);	// Compute the min obligate breaks from
				// marker i to marker j.
  int chr_knowns(int i);	// set the known count for a row

public:
  // Data

  // Functions
  map(void);

  istream& read(istream &in, int which);

  int rows() {return n;};
  int cols() {return m;};

  int mrk_knowns(int i);	// Compute the known count for a marker
  int mrk_retained (int i);	// Compute the retention count for a marker

  int setpair(int i, int j, int score);
  int getpair(int i, int j) {return Pair[i][j];};

  int NRL_score(int *);
  int NRL_score(int *, int);
  int RL_score(int *);
  int RL_score(int *, int);
  int bound(int *, int, int);

  marker& getmarker(int);
  chromosome& getchromosome(int);

  void pretty(int *);
  void pretty(int *, int);
};

#endif // MAP_H
