// $Id: rhclique.cc 1.3 Tue, 25 Feb 1997 12:34:46 -0400 humberto $
// Exact and aproximate routines for max weighted clique.
//
// Copyright (C) 1995 - 1997 Humberto Ortiz Zuazaga and Rosemarie Plaetke.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// See the file licence.html for a copy of the GNU GPL

#include <stdlib.h>
#include <fstream.h>
#include <GetOpt.h>		// The GNU C++ version.

#include "marker.h"
#include "chromosome.h"
#include "map.h"
#include "brkgraph.h"

int rdfs(map *, int *, int, int, int, int);


void usage(void) {
  cout << "wclique - Select markers for mapping.\n"
    "Copyright (C) 1995 - 1997 Humberto Ortiz Zuazaga and Rosemarie Plaetke.\n"
    "\n"
    "Usage: wclique [options] {input file}\n"
    "\tor wclique -h for help" << endl; 
  exit(1);
}

void help(void) {
  cout << "wclique - Select markers for mapping.\n"
    "Copyright (C) 1995 - 1997 Humberto Ortiz Zuazaga and Rosemarie Plaetke.\n"
    "\n"
    "Usage: wclique [-r -b breaks -e exact -i iters] {input file}" << endl; 
  cout << 
    " -r   -- use retained markers instead of informative markers.\n"
    " -b # -- specify the minimum break threshold.\n"
    " -e # -- specify the maximum number of markers to search exhaustively.\n"
    " -i # -- specify the number of iterations.\n";

  exit(0);
}

int main (int argc, char *argv[]) {
  map mapdata;
  int minbrk = 1;
  int which = KNOWNS;
  int exactlim = 50;
  int iterations = 100;

  GetOpt getopt (argc, argv, "rb:e:i:?h");
  int option_char;
  
  while ((option_char = getopt ()) != EOF) {
    switch (option_char) {
    case 'r': 
      cout << "Using retained weight function" << endl;
      which = RETAINED; 
      break;
    case 'b':
      minbrk = atoi(getopt.optarg);
      break;
    case 'e':
      exactlim = atoi(getopt.optarg);
      cout << "Exact limit set to " << exactlim << endl;
      break;
    case 'i':
      iterations = atoi(getopt.optarg);
      cout << "Performing " << iterations << " iterations" << endl;
      break;
    case '?': 
    case 'h':
      help();
      break;
    default:
      usage();
    }
  }

  if (argc > getopt.optind) {
    ifstream in(argv[getopt.optind]);
    if (!in) { 
      perror(argv[getopt.optind]);
      exit (1);
    }
    mapdata.read(in, which);
  } else {
    usage();
  }
  if (++getopt.optind != argc) {
    usage();
  }

  cout << "Minimum number of breaks = " << minbrk << endl;

  int best = 0;
  BrkGraph BG = BrkGraph(mapdata, minbrk);
  int tlen = BG.number_of_nodes();
  int *test = new int[tlen];
  for (int i = 0; i < tlen; i++) {
    test[i] = i;
  }
  best = BG.XRLF_MIS(test, 0, tlen, best, iterations, exactlim, tlen / 5);
  delete [] test;
}
